@@ -36,9 +36,12 @@ class KernelManager(ConnectionFileMixin):
     This version starts kernels with Popen.
     """
 
+    _created_context = Bool(False)
+
     # The PyZMQ Context to use for communication with the kernel.
     context = Instance(zmq.Context)
     def _context_default(self):
+        self._created_context = True
         return zmq.Context()
 
     # the class to create with our `client` method
@@ -337,15 +340,24 @@ class KernelManager(ConnectionFileMixin):
                 self.log.debug("Kernel is taking too long to finish, killing")
                 self._kill_kernel()
 
-    def cleanup(self, connection_file=True):
+    def cleanup_resources(self, restart=False):
         """Clean up resources when the kernel is shut down"""
-        if connection_file:
+        if not restart:
             self.cleanup_connection_file()
 
         self.cleanup_ipc_files()
         self._close_control_socket()
         self.session.parent = None
 
+        if self._created_context and not restart:
+            self.context.destroy(linger=100)
+
+    def cleanup(self, connection_file=True):
+        """Clean up resources when the kernel is shut down"""
+        warnings.warn("Method cleanup(connection_file=True) is deprecated, use cleanup_resources(restart=False).",
+                      FutureWarning)
+        self.cleanup_resources(restart=not connection_file)
+
     def shutdown_kernel(self, now=False, restart=False):
         """Attempts to stop the kernel process cleanly.
 
@@ -376,7 +388,13 @@ class KernelManager(ConnectionFileMixin):
             # most 1s, checking every 0.1s.
             self.finish_shutdown()
 
-        self.cleanup(connection_file=not restart)
+        from . import __version__
+        from distutils.version import LooseVersion
+
+        if LooseVersion(__version__) < LooseVersion('6.2'):
+            self.cleanup(connection_file=not restart)
+        else:
+            self.cleanup_resources(restart=restart)
 
     def restart_kernel(self, now=False, newports=False, **kw):
         """Restarts a kernel with the arguments that were used to launch it.
@@ -512,6 +530,7 @@ class KernelManager(ConnectionFileMixin):
 class AsyncKernelManager(KernelManager):
     """Manages kernels in an asynchronous manner """
 
+    client_class = DottedObjectName('jupyter_client.asynchronous.AsyncKernelClient')
     client_factory = Type(klass='jupyter_client.asynchronous.AsyncKernelClient')
 
     async def _launch_kernel(self, kernel_cmd, **kw):
@@ -590,7 +609,13 @@ class AsyncKernelManager(KernelManager):
             # most 1s, checking every 0.1s.
             await self.finish_shutdown()
 
-        self.cleanup(connection_file=not restart)
+        from . import __version__
+        from distutils.version import LooseVersion
+
+        if LooseVersion(__version__) < LooseVersion('6.2'):
+            self.cleanup(connection_file=not restart)
+        else:
+            self.cleanup_resources(restart=restart)
 
     async def restart_kernel(self, now=False, newports=False, **kw):
         """Restarts a kernel with the arguments that were used to launch it.
@@ -644,7 +669,7 @@ class AsyncKernelManager(KernelManager):
                 if hasattr(signal, 'SIGKILL'):
                     await self.signal_kernel(signal.SIGKILL)
                 else:
-                    await self.kernel.kill()
+                    self.kernel.kill()
             except OSError as e:
                 # In Windows, we will get an Access Denied error if the process
                 # has already terminated. Ignore it.
